/**
 * JavaScript for SRF portfolio format using the semanticFormats namespace
 * @see http://www.semantic-mediawiki.org/wiki/Help:portfolio format
 */
( function( $, srf ) {

	// Strict mode eliminates some JavaScript pitfalls
	'use strict';

	// Ensure the namespace is initialized and available
	srf.formats = srf.formats || {};

	/**
	 * Base constructor for objects representing a portfolio instance
	 * @since 1.8
	 * @type Object
	 */

	srf.formats.portfolio = function() {};

	srf.formats.portfolio.prototype = {
		// Specification of functions and parameters
		show: function( context ) {
			return context.each( function() {
				var	w = $( this ).width(),
					h = $( this ).height(),
					chart  = $( this ).find( '.container' ),
					portfolioID   = chart.attr( 'id' ),
					json   = mw.config.get( portfolioID );

				util.spinner.hide( { context: $( this ) } );
				$( this ).css( 'width', w ).css( 'height', h);


				var container = typeof json === 'string' ? jQuery.parseJSON( json ) : json;
				var caliber  = container.parameters.caliber;

				//Set padding
				var padding = w/10;
				
				//Create scale functions
				var xScale = d3.scale.linear()
				    .domain([0, 4])
				    .range([padding, w - padding ]);

				var yScale = d3.scale.linear()
				    .domain([0, 4])
				    .range([h - padding , padding]);

				var rScale = d3.scale.linear()
				    .domain([0, 4])
				    .range([w/20, h/10]);

				//Define X axis
				var xAxis = d3.svg.axis()
				    .scale(xScale)
				    .orient("bottom")
				    .ticks(4);

				//Define Y axis
				var yAxis = d3.svg.axis()
				    .scale(yScale)
				    .orient("left")
				    .ticks(4);

				//Prepare three sectors to fill with color   
				var sector1 = [{
					     "x": w - padding,
					     "y": padding
					 }, {
					    "x": w - padding,
					    "y": (h - 2*padding)/4*3 + padding
					 }, {
					    "x": (w - 2*padding)/4*1 + padding,
					    "y": padding
					 }, {
					    "x": w - padding,
					    "y": padding
					 }];

				var sector2 = [{
					    "x": (w - 2*padding)/4*1 + padding,
					    "y": padding
					 }, {
					    "x": padding,
					    "y": padding
					 }, {
					    "x": padding,
					    "y": (h - 2*padding)/4*1 + padding
					 }, {
					    "x": (w - 2*padding)/4*1 + padding,
					    "y": h/2
					 }, {
					    "x": (w - 2*padding)/4*3 + padding,
					    "y": h/2
					 }, {
					 	"x": (w - 2*padding)/4*1 + padding,
					 	"y": padding
					 }];

				var sector3 = [{
					    "x": (w - 2*padding)/4*1 + padding,
					    "y": h/2
					 }, {
					    "x": (w - 2*padding)/4*3 + padding,
					    "y": h/2
					 }, {
					    "x": w - padding,
					    "y": (h - 2*padding)/4*3 + padding
					 }, {
					    "x": w - padding,
					    "y": h - padding
					 }, {
					    "x": (w - 2*padding)/4*3 + padding,
					    "y": h - padding
					 }, {
					 	"x": (w - 2*padding)/4*1 + padding,
					    "y": h/2
					 }];
			
				//Accessor function 
				var lineFunction = d3.svg.line()
				 		.x(function (d) { return d.x; })
				 		.y(function (d) { return d.y; })
				 		.interpolate("linear");

				//Create SVG element
				var svg = d3.select("#" + portfolioID)
				    	.append("svg")
				    	.attr("width", w + 200)
				    	.attr("height", h + 50);

				//Fill sectors with color   
				svg.append("path")
					 	.attr("d", lineFunction(sector1))
					 	.attr("stroke", "black")
					 	.attr("stroke-width", 1)
					 	.attr("fill", "#99CCFF");

				svg.append("path")
					 	.attr("d", lineFunction(sector2))
					 	.attr("stroke", "black")
					 	.attr("stroke-width", 1)
					 	.attr("fill", "#99FFFF");

				svg.append("path")
					 	.attr("d", lineFunction(sector3))
					 	.attr("stroke", "black")
					 	.attr("stroke-width", 1)
					 	.attr("fill", "#CCFFFF");	 

				//Draw typical diagonal lines in portfolio matrix
				svg.append("line")        
					    .style("stroke", "black")
					    .style("opacity", 1)  
					    .attr("x1", (((h - 2*padding)/4)*1) + padding)     
					    .attr("y1", padding)      
					    .attr("x2", w - padding)     
					    .attr("y2", (((w - 2*padding)/4)*3) + padding);    
				
				svg.append("line")        
					    .style("opacity", 1)     
					    .style("stroke", "black")
					    .attr("x1", padding)     
					    .attr("y1", (((h - 2*padding)/4)*1) + padding)      
					    .attr("x2", (((w - 2*padding)/4)*3) + padding)     
					    .attr("y2", h - padding);
				       
				//Create grid
				//Draw horizontal lines
				for (var j=0; j <= 3; j++) {
				svg.append("line")
					.style("stroke", "black")
					.style("opacity", 1)
			        .attr("x1", padding)
			        .attr("y1", (((h - 2*padding)/4)*j) + padding)
			        .attr("x2", w - padding)
			        .attr("y2", (((h - 2*padding)/4)*j) + padding)
			        //.attr("stroke-width", 0.1)
			        ;
			    };

			    //Draw vertical lines
			    for (var j=1; j <= 4; j++) {
			    svg.append("line")
			    	.style("stroke", "black")
			    	.style("opacity", 1)
			        .attr("x1", (((w - 2*padding)/4)*j) + padding)
			        .attr("y1", padding)
			        .attr("x2", (((w - 2*padding)/4)*j) + padding)
			        .attr("y2", h - padding);
			    };

				//Create circles
				svg.selectAll("circle")
				    .data(container.data)
				    .enter()
				    .append("circle")
				    .style("opacity" , .7)
				    .attr("cx", function (d) { return xScale((d.value[0])/25); })
				    .attr("cy", function (d) { return yScale((d.value[1])/25); })
				    .attr("r", function (d) { return rScale(d.value[2]); })
				    .attr("stroke", "black")
		            .attr("fill", "#0000FF")
		            .on('mouseover', function(d){
	    				var nodeSelection = d3.select(this).style({opacity:'1.0'});
	    				nodeSelection.select("text").style({opacity:'1.0'});
					})
					.on('mouseout', function(d){
	      				d3.select(this).style({opacity:'.7',})
	      				d3.select("text").style({opacity:'1.0'});
	    			});
	            

			
				//Create labels
				svg.selectAll("text")
				   .data(container.data)
				   .enter()
				   .append("text")
				   .text(function(d) { return d.label; })
				   .attr("x", function(d) { return xScale((d.value[0])/25) })
				   .attr("y", function(d) { return yScale((d.value[1])/25); })
				   .attr("font-family", "sans-serif")
				   .attr("font-size", "11px")
				   .attr("fill", "white")
				   .attr("dy", ".35em")	
				   .attr("text-anchor", "middle");
				   
			  	

				//Create X axis
				svg.append("g")
					.style("stroke", "black")
					.style("opacity", 1)
				    .attr("class", "axis")
				    .attr("transform", "translate(0," + (h - padding) + ")")
				    .call(xAxis);
				    

				//Create Y axis
				svg.append("g")
					.style("stroke", "black")
					.style("opacity", 1)
				    .attr("class", "axis")
				    .attr("transform", "translate(" + padding + ")")
				    .call(yAxis);


				//Axis text and legend
				svg.append("text")
	    				.attr("class", "x label")
	    				.attr("text-anchor", "middle")
	    				.attr("x", (w - 2*padding)/4*2 + padding)
	    				.attr("y", h)
	    				.attr("font-size", w/30)
	    				.text("Ressourcenstärke");
		    		
	    		svg.append("text")
	    				.attr("class", "y label")
	    				.attr("text-anchor", "end")
	    				.attr("x", padding/2)
	    				.attr("y", (h - 2*padding)/4*2 + padding)
	    				.attr("font-size", w/30)
	    				.attr("transform", "translate(" +(-4.75*padding)+ "," +(h/2 - 1.2*padding)+ ") rotate(-90)")
	    				.text("Technologieattraktivität");
				
	    		//Prepare legend display
	    		var LegendOptions1 = ['Investieren','Desinvestieren','Selektionsfeld1', 'Selektionsfeld2'];		
				var LegendOptions2 = ['0','1','2','3','4'];
				var LegendOptions3 = ['Sehr niedrig', 'Niedrig', 'Mittel', 'Hoch', 'Sehr hoch'];
				var colorscale = ["#99CCFF",'white','#99FFFF','#CCFFFF'];

				//Initiate Legend	
				var legend1 = svg.append("g")
				    .attr("class", "legend")
					
				//Create colored squares
				legend1.selectAll('rect')
				    .data(LegendOptions1)
				    .enter()
				    .append("rect")
				    .attr("x", w+30 - padding/2)
				    .attr("y", function (d, i) {
	    				return i * padding/2 + padding;
					})
				    .attr("stroke", "black")
				    .attr("width", 10)
				    .attr("height", 10)
				    //.style("fill", "#99CCFF");
				    .style("fill", function (d, i) {
	    				return colorscale[i];
					});
		
				//Create text next to squares
				legend1.selectAll('text')
				    .data(LegendOptions1)
				    .enter()
				    .append("text")
				    .attr("x", w+30 - padding/2 + 12)
				    .attr("y", function (d, i) {
	    				return i * padding/2 + padding + 9 ;
					})
				    //.attr("dy", ".35em")	
				   	.attr("text-anchor", "front")
				    .attr("font-size", "11px")
				    .attr("fill", "#737373")
				    .text(function (d) {
				    return d;
				});
			
				//Text for axis ticks
				var legend2 = svg.append("g")
				    .attr("class", "legend")    

				
				legend2.selectAll('text')
				    .data(LegendOptions2)
				    .enter()
				    .append("text")
				    .attr("x", w+30 - padding/2 )
				    .attr("y", function (d, i) {
	    				return i * padding/2 + 2.5*padding + 50;
					})
				   	.attr("text-anchor", "front")
				    .attr("font-size", "11px")
				    .attr ("stroke", "black")
				    .text(function (d) {
				    return d;

				});
					  
				var legend3 = svg.append("g")
				    .attr("class", "legend")

				legend3.selectAll('text')
				    .data(LegendOptions3)
				    .enter()
				    .append("text")
				    .attr("x", w+30 - padding/2 + 12  )
				    .attr("y", function (d, i) {
	    				return i * padding/2 + 2.5*padding + 50;
					})
				    //.attr("dy", ".35em")	
				   	.attr("text-anchor", "front")
				    .attr("font-size", "11px")
				    .attr("fill", "#737373")
				    .text(function (d) { return d; });
		
	
			    if (caliber != '') {    
			        var calibersymbol = svg.append("text")
			            .attr("x", w+30 - padding/2 )
			            .attr("y", padding*6.3 )
			            .attr("dy", "-0.25em") 
			            .attr("text-anchor", "front")
			            .attr("font-size", "12px")
			            .attr("stroke", "black")
			            .text("Ø");
			            
			        var calibertext = svg.append("text")
			            .attr("x", w+30 - padding/2 +12 )
			            .attr("y", padding*6.3 )
			            .attr("dy", "-0.25em") 
			            .attr("text-anchor", "front")
			            .attr("font-size", "11px")
			            .attr("fill", "#737373")
			            .text(""+caliber+"");
				};
			})
		}
	}

	// Create class instance
	var portfolio = new srf.formats.portfolio();

	// Get access to SRF specific utilities function
	var util = new srf.util();

	$( document ).ready(function() {


		//alert('1.');
		// Use the class selector to find all instances relevant to the "portfolio" printer
		// since a wiki page can have more than one instance of the same result printer
		// .each() ensures instances are handled separately
		$( '.srf-portfolio' ).each(function() {

			// Access methods available through the portfolio class
			portfolio.show( $( this ) );
		
		} );
	} );
} )( jQuery, semanticFormats );